@@ -119,7 +119,7 @@ def submit_python_job_file(
         n_calls_to_submit_fn[0] += 1
 
     def submit_now_fn():
-        session.submit(cli, njobsmax=njobsmax)
+        return session.submit(cli, njobsmax=njobsmax)
 
     exec_scope = {
         "qondor": qondor,
@@ -303,7 +303,7 @@ class Session(object):
         whitelist = self.htcondor_settings.pop("cmsconnect_whitelist", None)
         if os.uname()[1] == "login.uscms.org" or os.uname()[1] == "login-el7.uscms.org":
             qondor.logger.warning("Detected CMS Connect; loading specific settings")
-            cmsconnect_settings(
+            qondor.cmsconnect.cmsconnect_settings(
                 self.htcondor_settings,
                 cli=cli,
                 blacklist=blacklist,
@@ -578,6 +578,7 @@ class Session(object):
         # Clear the submittables
         self.submittables = []
         self.submitted = []
+        return submission
 
 
 class Cluster(object):
@@ -607,9 +608,33 @@ class Cluster(object):
         self.scope = {} if scope is None else scope
         self.htcondor = {} if htcondor is None else htcondor
         self.run_args = run_args
+
+        # Figure out the run environment
+        self._is_conda_pack = False
         if qondor.utils.is_string(run_env):
-            run_env = RUN_ENVS[run_env]
-        self.run_env = run_env
+            if run_env.startswith("condapack:"):
+                # Conda pack mode
+                conda_tarball = run_env.replace("condapack:", "", 1)
+                self.run_env = []
+                if seutils.has_protocol(conda_tarball):
+                    self.run_env.append("xrdcp {0} .".format(conda_tarball))
+                else:
+                    raise NotImplementedError(
+                        "transferable conda tarballs are not supported yet"
+                    )
+                self.run_env += [
+                    'export PYTHONPATH=""',
+                    "mkdir -p my_env",
+                    "tar -xzf {0} -C my_env".format(osp.basename(conda_tarball)),
+                    "source my_env/bin/activate",
+                    "conda-unpack",
+                ]
+                self._is_conda_pack = True
+            elif run_env in RUN_ENVS:
+                self.run_env = RUN_ENVS[run_env]
+            else:
+                raise ValueError("Cannot interpret environment '{0}'".format(run_env))
+
         self.transfer_files = {}
         if transfer_files:
             for f in transfer_files:
@@ -650,7 +675,7 @@ class Cluster(object):
             return False
 
         if not check_if_in_pips("qondor"):
-            pips.append(("qondor", "auto"))
+            pips.append(("qondor=={}".format(qondor.version), "auto"))
         if not check_if_in_pips("seutils"):
             pips.append(("seutils", "auto"))
         for pip in pips:
@@ -754,23 +779,31 @@ class Cluster(object):
         ]
         # Set the runtime environment (typically sourcing scripts to get the right python/gcc/ROOT/etc.)
         sh += self.run_env + [""]
-        # Set up a directory to install python packages in, and put on the path
-        # Currently requires $pipdir to be defined... might want to figure out something more clever
-        sh += [
-            "set -uxoE pipefail",
-            'echo "Setting up custom pip install dir"',
-            'HOME="$(pwd)"',
-            'export pip_install_dir="$(pwd)/install"',
-            'mkdir -p "${pip_install_dir}/bin"',
-            'mkdir -p "${pip_install_dir}/lib/python2.7/site-packages"',
-            'export PATH="${pip_install_dir}/bin:${PATH}"',
-            "export PYTHONVERSION=$(python -c \"import sys; print('{}.{}'.format(sys.version_info.major, sys.version_info.minor))\")",
-            'export PYTHONPATH="${pip_install_dir}/lib/python${PYTHONVERSION}/site-packages:${PYTHONPATH}"',
-            "",
-            "pip -V",
-            "which pip",
-            "",
-        ]
+        sh += ["set -uxoE pipefail"]
+
+        if not self._is_conda_pack:
+            # Set up a directory to install python packages in, and put on the path
+            # Currently requires $pipdir to be defined... might want to figure out something more clever
+            sh += [
+                'echo "Setting up custom pip install dir"',
+                'HOME="$(pwd)"',
+                'export pip_install_dir="$(pwd)/install"',
+                'export PATH="${pip_install_dir}/bin:${PATH}"',
+                "export PYTHONVERSION=$(python -c \"import sys; print('{}.{}'.format(sys.version_info.major, sys.version_info.minor))\")",
+                'export PYTHONPATH="${pip_install_dir}/lib/python${PYTHONVERSION}/site-packages:${PYTHONPATH}"',
+                'mkdir -p "${pip_install_dir}/bin"',
+                'mkdir -p "${pip_install_dir}/lib/python${PYTHONVERSION}/site-packages"',
+                "",
+                "pip -V",
+                "which pip",
+                "",
+            ]
+
+        if self._is_conda_pack:
+            pip_install_options = "--no-cache-dir --no-use-pep517"
+        else:
+            pip_install_options = '--install-option="--prefix=${{pip_install_dir}}" --no-cache-dir --no-use-pep517'
+
         # `pip install` the required pip packages for the job
         for package, install_instruction in self.pips:
             package_name, version_stuff = qondor.utils.pip_split_version(
@@ -792,18 +825,12 @@ class Cluster(object):
                     [
                         "mkdir {0}".format(package),
                         "tar xf {0}.tar -C {0}".format(package),
-                        'pip install --install-option="--prefix=${{pip_install_dir}}" --no-cache-dir -e {0}/'.format(
-                            package
-                        ),
+                        "pip install {1} -e {0}/".format(package, pip_install_options),
                     ]
                 )
             else:
                 # Non-editable install from pypi
-                sh.append(
-                    'pip install --install-option="--prefix=${{pip_install_dir}}" --no-cache-dir {0}'.format(
-                        package
-                    )
-                )
+                sh.append("pip install {1} {0}".format(package, pip_install_options))
         # Make the actual python call to run the required job code
         # Also echo the exitcode of the python command to a file, to easily check whether jobs succeeded
         # First compile the command - which might take some command line arguments
@@ -837,81 +864,3 @@ class Cluster(object):
             "session", Session()
         )  # Make new session if not set by keyword
         session.submit(self, *args, **kwargs)
-
-
-def cmsconnect_get_all_sites():
-    """
-    Reads the central config for cmsconnect to determine the list of all available sites
-    """
-    try:
-        from configparser import RawConfigParser  # python 3
-    except ImportError:
-        import ConfigParser  # python 2
-
-        RawConfigParser = ConfigParser.RawConfigParser
-    cfg = RawConfigParser()
-    cfg.read("/etc/ciconnect/config.ini")
-    sites = cfg.get("submit", "DefaultSites").split(",")
-    all_sites = set(sites)
-    return all_sites
-
-
-def cmsconnect_settings(sub, blacklist=None, whitelist=None, cli=False):
-    """
-    Adds special cmsconnect settings to submission dict in order to submit
-    on cmsconnect via the python bindings, or in case of submitting by the
-    command line, to set the DESIRED_Sites key.
-    Modifies the dict in place.
-    """
-    all_sites = cmsconnect_get_all_sites()
-
-    # Check whether the user whitelisted or blacklisted some sites
-    desired_sites = None
-    if blacklist or whitelist:
-        import fnmatch
-
-        blacklisted = []
-        whitelisted = []
-        # Build the blacklist
-        if blacklist:
-            for blacksite_pattern in blacklist:
-                for site in all_sites:
-                    if fnmatch.fnmatch(site, blacksite_pattern):
-                        blacklisted.append(site)
-        # Build the whitelist
-        if whitelist:
-            for whitesite_pattern in whitelist:
-                for site in all_sites:
-                    if fnmatch.fnmatch(site, whitesite_pattern):
-                        whitelisted.append(site)
-        # Convert to list and sort
-        blacklisted = list(set(blacklisted))
-        blacklisted.sort()
-        whitelisted = list(set(whitelisted))
-        whitelisted.sort()
-        logger.info("Blacklisting: %s", ",".join(blacklisted))
-        logger.info("Whitelisting: %s", ",".join(whitelisted))
-        desired_sites = list(
-            (set(all_sites) - set(blacklisted)).union(set(whitelisted))
-        )
-        desired_sites.sort()
-
-    # Add a plus only if submitting via .jdl file
-    def addplus(key):
-        return "+" + key if cli else key
-
-    if desired_sites:
-        logger.info("Submitting to desired sites: %s", ",".join(desired_sites))
-        sub[addplus("DESIRED_Sites")] = '"' + ",".join(desired_sites) + '"'
-    else:
-        logger.info("Submitting to all sites: %s", ",".join(all_sites))
-    if not cli:
-        sub[addplus("ConnectWrapper")] = '"2.0"'
-        sub[addplus("CMSGroups")] = '"/cms,T3_US_FNALLPC"'
-        sub[addplus("MaxWallTimeMins")] = "500"
-        sub[addplus("ProjectName")] = '"cms.org.fnal"'
-        sub[addplus("SubmitFile")] = '"irrelevant.jdl"'
-        sub[addplus("AccountingGroup")] = '"analysis.{0}"'.format(os.environ["USER"])
-        logger.warning(
-            "FIXME: CMS Connect settings currently hard-coded for a FNAL user"
-        )
